#![cfg(test)]
#![allow(dead_code)]
pub(crate) const COIN_COMMAND: &str = include_str!("coin.json");
pub(crate) const ASCSET_HELP_COMMAND: &str = include_str!("ascset_help.json");
pub(crate) const SUMMARY_COMMAND: &str = include_str!("summary.json");
pub(crate) const STATS_COMMAND: &str = include_str!("stats.json");
pub(crate) const LITESTATS_COMMAND: &str = include_str!("litestats.json");
pub(crate) const POOLS_COMMAND: &str = include_str!("pools.json");
pub(crate) const DEVS_COMMAND: &str = include_str!("devs.json");
pub(crate) const DEVDETAILS_COMMAND: &str = include_str!("devdetails.json");
pub(crate) const CHECK_COMMAND: &str = include_str!("check.json");
pub(crate) const VERSION_COMMAND: &str = include_str!("version.json");
pub(crate) const CONFIG_COMMAND: &str = include_str!("config.json");
pub(crate) const ASCSET_WORKMODE_COMMAND: &str = include_str!("ascset_workmode.json");
pub(crate) const ASCSET_SOFTON_COMMAND: &str = include_str!("ascset_softon.json");
pub(crate) const ASCSET_SOFTOFF_COMMAND: &str = include_str!("ascset_softoff.json");
pub(crate) const PARSED_STATS_COMMAND: &str = include_str!("stats_parsed.json");
pub(crate) const AVALON_A_STATS: &str = include_str!("avalon_a_stats.json");
pub(crate) const AVALON_A_STATS_PARSED: &str = include_str!("avalon_a_stats_parsed.json");
