#![cfg(test)]
#![allow(dead_code)]

pub(crate) const CONFIG: &str = include_str!("config.json");
pub(crate) const DEVS: &str = include_str!("devs.json");
pub(crate) const FANS: &str = include_str!("fans.json");
pub(crate) const POOLS: &str = include_str!("pools.json");
pub(crate) const POWER: &str = include_str!("power.json");
pub(crate) const PROFILES: &str = include_str!("profiles.json");
pub(crate) const STATS: &str = include_str!("stats.json");
pub(crate) const SUMMARY: &str = include_str!("summary.json");
pub(crate) const TEMPS: &str = include_str!("temps.json");
pub(crate) const VERSION: &str = include_str!("version.json");

pub(crate) const VOLTAGEGET_0: &str = include_str!("voltageget_0.json");
pub(crate) const VOLTAGEGET_1: &str = include_str!("voltageget_1.json");
pub(crate) const VOLTAGEGET_2: &str = include_str!("voltageget_2.json");
pub(crate) const HEALTHCHIPGET_0: &str = include_str!("healthchipget_0.json");
pub(crate) const HEALTHCHIPGET_1: &str = include_str!("healthchipget_1.json");
pub(crate) const HEALTHCHIPGET_2: &str = include_str!("healthchipget_2.json");
