#![cfg(test)]
#![allow(dead_code)]

pub(crate) const AM_DEVS: &str = include_str!("devs.json");
pub(crate) const AM_STATS: &str = include_str!("stats.json");
pub(crate) const AM_POOLS: &str = include_str!("pools.json");
pub(crate) const AM_SUMMARY: &str = include_str!("summary.json");
pub(crate) const AM_VERSION: &str = include_str!("version.json");
