#![cfg(test)]
#![allow(dead_code)]

pub(crate) const SUMMARY: &str = include_str!("summary.json");
pub(crate) const CAPABILITIES: &str = include_str!("capabilities.json");
pub(crate) const TEMPS: &str = include_str!("temps.json");
pub(crate) const NETWORK: &str = include_str!("network.json");
pub(crate) const CHIP_CLOCKS: &str = include_str!("chip_clocks.json");
pub(crate) const CHIP_TEMPS: &str = include_str!("chip_temps.json");
pub(crate) const CHIP_VOLTAGES: &str = include_str!("chip_voltages.json");
pub(crate) const CHIP_HASHRATES: &str = include_str!("chip_hashrates.json");
