//! Miner specific implementations of control functionality and data gathering.

pub mod antminer;
pub mod avalonminer;
pub mod bitaxe;
pub mod braiins;
pub mod epic;
pub mod luxminer;
pub mod marathon;
pub mod traits;
pub mod vnish;
pub mod whatsminer;
